//
//  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//  N o t e b o o k   H a r d w a r e   C o n t r o l   A C P I   D L L
//  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//
// Fan control for Asus F3JP
// User class by Martin Nssing

// The Fan speed will be controlled by the _SB.PCI0.SBRG.EC0.WMFN register in the Embedded Controller.
// WMFN is a 8 bit register. The MSB (Bit 7) control the method of fan control:
//  manual = 0, automatic (default)= 1
// The next 4 Bit (Bit 6 - 2) sets the fan speed (only when Bit 7 is 0):
//  value 0x0 to 0x5: fan is off
//  from value 0x5 to 0xF: fan speeds in ascending order
//  The only problem is that a maximum fan speed set in WMFN(0xF) is not the realy maximum speed of the fan. In automatic mode,
//  the fan can drive the motor more faster on higher tempertures!
//
// The Fan control of this class has 5 "temperatures on values" with the corresponding fan speed 0-9.
// "FAN 0 speed" means the fan speed seated on temperatures under "FAN ON 1 LEVEL".
// If the fan speed is set to 10, the program goes in automatic mode and allows to drive the fan very fast
// (a little workaround for the problem mentioned above, but only for the appropriate temperature window).
// The control class allows also to set 5 "temperatures off values" to realize a hysteresis.
// The temperatures must are in ascendent order! The class not check this.
// The class is not realy tested, please feel free to fix or expand the functionality

//****NOTE!!:IF YOU TURN YOUR FAN OFF MANUALLY, OVERHEATING MAY OCCUR.!!****


public static class ASUSTeK
{

    #region Notebook Asus F3JP : class F3JP

    public static class F3JP
    {

        #region Class for Asus F3JC LCD Brightness control : class BRIGHTNESS
        //
        // The class must have the name BRIGHTNESS
        //
        public static class BRIGHTNESS
        {
            // Default constants for the brightness class
            public const string description = "LCD Brightness Control";             // Description of current class
            public const bool default_enable = false;                                // true = By default the current class is enabled in NHC
            public const int icon = 1;                                              // 0 = Hardware icon, 1 = Brightness icon, 2 = Fan icon, 3 = Battery icon, 4 = Audio icon

            // Constants for the brightness control (NHC will need this constants)
            public const int minimum_brightness_level = 0;
            public const int maximum_brightness_level = 15;
            public const int brightness_level_step = 1;

            #region Value class for brightness_level : class BRIGHTNESS_LEVEL
            //
            public static class BRIGHTNESS_LEVEL
            {
                // Properties of the value class
                public const string description = "Current LCD brightness level";   // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int brightness_level = 0;                             // It must have the same name as the class (case are ignored)
            }
            //
            #endregion

            // Function to set the brightness (it must have the name set_brightness)
            public static bool set_brightness(int new_level)
            {
                return true;//ACPI.METHOD.SetValue("_SB.ATKD.SPLV", new_level);
            }

            // Function to get and refresh the brightness level (it must have the name set_brightness)
            public static bool get_brightness(ref int level)
            {
                bool result = true;//ACPI.METHOD.GetValue("_SB.ATKD.GPLV", ref level);     // get the brightness level
                BRIGHTNESS_LEVEL.brightness_level = level;
                return result;
            }
        }
        //
        #endregion


        #region Class for Asus F3JC FAN control : class FAN
        //
        public static class FAN
        {
            // Info
            public const string description = "Notebook FAN Control";               // Description of current class
            public const bool default_enable = true;                                // true = By default the current class is enabled in NHC
            public const int icon = 2;                                              // 0 = Hardware icon, 1 = Brightness icon, 2 = Fan icon, 3 = Battery icon, 4 = Audio icon
			// Global variables for FAN control
			public static int actual_level = -1;
            public const int refresh_interval = 1;
            public const int MAX_SPEED_NR = 5;

			// Global fan control data
			public static int[] speed = new int[MAX_SPEED_NR + 1];
			public static int[] on_temp = new int[MAX_SPEED_NR + 2];
			public static int[] off_temp = new int[MAX_SPEED_NR + 2];

            #region Value class for FAN_0_SPEED : class FAN_0_SPEED
            //
            public static class FAN_0_SPEED
            {
                // Properties of the value class
                public const string description = "FAN 0 speed";                  // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int fan_0_speed = 0;                              // It must have the same name as the class (case are ignored)
                public const string unit = "";                                 // unit of value

                // With this class it is possible to change the value in NHC on runtime
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 0;                       // minimum input value
                    public const int nhc_maximum_value = 10;                       // maximum input value
                    public const int nhc_default_value = 0;                       // default value
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {
						// reset actual level
						actual_level = -1;
						speed[0] = fan_0_speed;
                    	return true;
                    }
                }
            }
            //
            #endregion

            #region Value class for FAN_1_SPEED : class FAN_1_SPEED
            //
            public static class FAN_1_SPEED
            {
                // Properties of the value class
                public const string description = "FAN 1 speed";                  // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int fan_1_speed = 1;                              // It must have the same name as the class (case are ignored)
                public const string unit = "";                                 // unit of value

                // With this class it is possible to change the value in NHC on runtime
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 0;                       // minimum input value
                    public const int nhc_maximum_value = 10;                       // maximum input value
                    public const int nhc_default_value = 1;                       // default value
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {
						// reset actual level
						actual_level = -1;
						speed[1] = fan_1_speed;
                    	return true;
                    }
                }
            }
            //
            #endregion

            #region Value class for FAN_2_SPEED : class FAN_2_SPEED
            //
            public static class FAN_2_SPEED
            {
                // Properties of the value class
                public const string description = "FAN 2 speed";               // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int fan_2_speed = 3;                           // It must have the same name as the class (case are ignored)
                public const string unit = "";                                 // unit of value

                // With this class it is possible to change the value in NHC on runtime
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 0;                       // minimum input value
                    public const int nhc_maximum_value = 10;                       // maximum input value
                    public const int nhc_default_value = 3;                       // default value
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {
						// reset actual level
						actual_level = -1;
						speed[2] = fan_2_speed;
                    	return true;
                    }
                }
            }
            //
            #endregion

            #region Value class for FAN_3_SPEED : class FAN_3_SPEED
            //
            public static class FAN_3_SPEED
            {
                // Properties of the value class
                public const string description = "FAN 3 speed";                 // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int fan_3_speed = 6;                             // It must have the same name as the class (case are ignored)
                public const string unit = "";                                 // unit of value

                // With this class it is possible to change the value in NHC on runtime
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 0;                       // minimum input value
                    public const int nhc_maximum_value = 10;                       // maximum input value
                    public const int nhc_default_value = 6;                       // default value
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {
						// reset actual level
						actual_level = -1;
						speed[3] = fan_3_speed;
                    	return true;
                    }
                }
            }
            //
            #endregion

            #region Value class for FAN_4_SPEED : class FAN_4_SPEED
            //
            public static class FAN_4_SPEED
            {
                // Properties of the value class
                public const string description = "FAN 4 speed";                 // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int fan_4_speed = 9;                             // It must have the same name as the class (case are ignored)
                public const string unit = "";                                 // unit of value

                // With this class it is possible to change the value in NHC on runtime
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 0;                       // minimum input value
                    public const int nhc_maximum_value = 10;                       // maximum input value
                    public const int nhc_default_value = 9;                       // default value
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {
						// reset actual level
						actual_level = -1;
						speed[4] = fan_4_speed;
                    	return true;
                    }
                }
            }
            //
            #endregion

            #region Value class for FAN_5_SPEED : class FAN_5_SPEED
            //
            public static class FAN_5_SPEED
            {
                // Properties of the value class
                public const string description = "FAN 5 speed";                 // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int fan_5_speed = 9;                             // It must have the same name as the class (case are ignored)
                public const string unit = "";                                 // unit of value

                // With this class it is possible to change the value in NHC on runtime
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 0;                       // minimum input value
                    public const int nhc_maximum_value = 10;                       // maximum input value
                    public const int nhc_default_value = 10;                       // default value
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {
						// reset actual level
						actual_level = -1;
						speed[5] = fan_5_speed;
                    	return true;
                    }
                }
            }
            //
            #endregion



            #region  Value class for LEVEL_1_ON : class LEVEL_1_ON
            //
            public static class LEVEL_1_ON
            {
                // Properties of the value class
                public const string description = "FAN ON 1 level";               // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int level_1_on = 45;                                // It must have the same name as the class (case are ignored)
                public const string unit = "C";                                    // unit of value

                // With this class it is possible to change the value in NHC on runtime
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 0;                         // minimum input value
                    public const int nhc_maximum_value = 60;                        // maximum input value
                    public const int nhc_default_value = 45;                        // default value
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {
						// reset actual level
						actual_level = -1;
						on_temp[1] = level_1_on;
                    	return true;
                    }
                }
            }
            //
            #endregion

            #region  Value class for LEVEL_2_ON : class LEVEL_2_ON
            //
            public static class LEVEL_2_ON
            {
                // Properties of the value class
                public const string description = "FAN ON 2 level";            // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int level_2_on = 50;                             // It must have the same name as the class (case are ignored)
                public const string unit = "C";                                    // unit of value

                // With this class it is possible to change the value in NHC on runtime
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 30;                        // minimum input value
                    public const int nhc_maximum_value = 68;                        // maximum input value
                    public const int nhc_default_value = 50;                        // default value
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {
						// reset actual level
						actual_level = -1;
						on_temp[2] = level_2_on;
                    	return true;
                    }
                }
            }
            //
            #endregion

            #region  Value class for LEVEL_3_ON : class LEVEL_3_ON
            //
            public static class LEVEL_3_ON
            {
                // Properties of the value class
                public const string description = "FAN ON 3 level";              // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int level_3_on = 55;                               // It must have the same name as the class (case are ignored)
                public const string unit = "C";                                    // unit of value

                // With this class it is possible to change the value in NHC on runtime
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 35;                        // minimum input value
                    public const int nhc_maximum_value = 86;                        // maximum input value
                    public const int nhc_default_value = 55;                        // default value
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {
						// reset actual level
						actual_level = -1;
						on_temp[3] = level_3_on;
                    	return true;
                    }
                }
            }
            //
            #endregion

            #region  Value class for LEVEL_4_ON : class LEVEL_4_ON
            //
            public static class LEVEL_4_ON
            {
                // Properties of the value class
                public const string description = "FAN ON 4 level";              // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int level_4_on = 60;                               // It must have the same name as the class (case are ignored)
                public const string unit = "C";                                    // unit of value

                // With this class it is possible to change the value in NHC on runtime
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 36;                        // minimum input value
                    public const int nhc_maximum_value = 86;                        // maximum input value
                    public const int nhc_default_value = 60;                        // default value
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {
						// reset actual level
						actual_level = -1;
						on_temp[4] = level_4_on;
                    	return true;
                    }
                }
            }
            //
            #endregion

            #region  Value class for LEVEL_5_ON : class LEVEL_5_ON
            //
            public static class LEVEL_5_ON
            {
                // Properties of the value class
                public const string description = "FAN ON 5 level";              // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int level_5_on = 65;                               // It must have the same name as the class (case are ignored)
                public const string unit = "C";                                    // unit of value

                // With this class it is possible to change the value in NHC on runtime
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 38;                        // minimum input value
                    public const int nhc_maximum_value = 86;                        // maximum input value
                    public const int nhc_default_value = 65;                        // default value
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {
						// reset actual level
						actual_level = -1;
						on_temp[5] = level_5_on;
                    	return true;
                    }
                }
            }
            //
            #endregion

            #region  Value class for LEVEL_1_OFF : class LEVEL_1_OFF
            //
            public static class LEVEL_1_OFF
            {
                // Properties of the value class
                public const string description = "FAN OFF 1 level";               // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int level_1_off = 43;                                // It must have the same name as the class (case are ignored)
                public const string unit = "C";                                    // unit of value

                // With this class it is possible to change the value in NHC on runtime
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 0;                         // minimum input value
                    public const int nhc_maximum_value = 60;                        // maximum input value
                    public const int nhc_default_value = 43;                        // default value
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {
						// reset actual level
						actual_level = -1;
						off_temp[1] = level_1_off;
                    	return true;
                    }
                }
            }
            //
            #endregion

            #region  Value class for LEVEL_2_OFF : class LEVEL_2_OFF
            //
            public static class LEVEL_2_OFF
            {
                // Properties of the value class
                public const string description = "FAN OFF 2 level";            // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int level_2_off = 48;                             // It must have the same name as the class (case are ignored)
                public const string unit = "C";                                    // unit of value

                // With this class it is possible to change the value in NHC on runtime
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 30;                        // minimum input value
                    public const int nhc_maximum_value = 68;                        // maximum input value
                    public const int nhc_default_value = 48;                        // default value
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {
						// reset actual level
						actual_level = -1;
						off_temp[2] = level_2_off;
                    	return true;
                    }
                }
            }
            //
            #endregion

            #region  Value class for LEVEL_3_OFF : class LEVEL_3_OFF
            //
            public static class LEVEL_3_OFF
            {
                // Properties of the value class
                public const string description = "FAN OFF 3 level";              // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int level_3_off = 53;                               // It must have the same name as the class (case are ignored)
                public const string unit = "C";                                    // unit of value

                // With this class it is possible to change the value in NHC on runtime
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 35;                        // minimum input value
                    public const int nhc_maximum_value = 86;                        // maximum input value
                    public const int nhc_default_value = 53;                        // default value
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {
						// reset actual level
						actual_level = -1;
						off_temp[3] = level_3_off;
                    	return true;
                    }
                }
            }
            //
            #endregion

            #region  Value class for LEVEL_4_OFF : class LEVEL_4_OFF
            //
            public static class LEVEL_4_OFF
            {
                // Properties of the value class
                public const string description = "FAN OFF 4 level";              // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int level_4_off = 58;                               // It must have the same name as the class (case are ignored)
                public const string unit = "C";                                    // unit of value

                // With this class it is possible to change the value in NHC on runtime
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 36;                        // minimum input value
                    public const int nhc_maximum_value = 86;                        // maximum input value
                    public const int nhc_default_value = 58;                        // default value
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {
						// reset actual level
						actual_level = -1;
						off_temp[4] = level_4_off;
                    	return true;
                    }
                }
            }
            //
            #endregion

            #region  Value class for LEVEL_5_OFF : class LEVEL_5_OFF
            //
            public static class LEVEL_5_OFF
            {
                // Properties of the value class
                public const string description = "FAN OFF 5 level";              // Description of current value
                public const bool nhc_visible = true;                               // true -> value is visible in NHC

                // Value
                public static int level_5_off = 63;                               // It must have the same name as the class (case are ignored)
                public const string unit = "C";                                    // unit of value

                // With this class it is possible to change the value in NHC on runtime
                public static class NHC_WRITE
                {
                    public const bool nhc_write = true;                             // true -> it is possible to change the value in NHC
                    public const int nhc_minimum_value = 38;                        // minimum input value
                    public const int nhc_maximum_value = 86;                        // maximum input value
                    public const int nhc_default_value = 63;                        // default value
                    public static bool nhc_on_write()                               // Optional function - NHC will call this function if the user change the value on runtime and on NHC initialisation
                    {
						// reset actual level
						actual_level = -1;
						off_temp[5] = level_5_off;
                    	return true;
                    }
                }
            }
            //
            #endregion

			public static bool check_for_higher_speed_level(int start_index)
			{
				int i = start_index;
				bool result = false;

				while (!result && i <= MAX_SPEED_NR)
				{
					result = result || (speed[i] == actual_level);
					i++;
				}
				return result;
			}

            public static bool init()
            {
				speed[0] = FAN_0_SPEED.fan_0_speed;
				speed[1] = FAN_1_SPEED.fan_1_speed;
				speed[2] = FAN_2_SPEED.fan_2_speed;
				speed[3] = FAN_3_SPEED.fan_3_speed;
				speed[4] = FAN_4_SPEED.fan_4_speed;
				speed[5] = FAN_5_SPEED.fan_5_speed;

				on_temp[0] = 0;
				on_temp[1] = LEVEL_1_ON.level_1_on;
				on_temp[2] = LEVEL_2_ON.level_2_on;
				on_temp[3] = LEVEL_3_ON.level_3_on;
				on_temp[4] = LEVEL_4_ON.level_4_on;
				on_temp[5] = LEVEL_5_ON.level_5_on;
				off_temp[6] = 200;

				off_temp[0] = 0;
				off_temp[1] = LEVEL_1_OFF.level_1_off;
				off_temp[2] = LEVEL_2_OFF.level_2_off;
				off_temp[3] = LEVEL_3_OFF.level_3_off;
				off_temp[4] = LEVEL_4_OFF.level_4_off;
				off_temp[5] = LEVEL_5_OFF.level_5_off;
				off_temp[6] = 200;

				return true;
			}

            public static bool refresh()
            {
                bool result = false;
                bool find = false;
                bool previous_level = false;
                int temp = 0;
                int i = 0;

				// read actual CPU temp
				result = ACPI.FIELD.Read("_SB.PCI0.SBRG.EC0.ECPU", ref temp);

                while (!find && (i <= MAX_SPEED_NR))
                {
					// Check, if the previous temp was higher then the actual
					previous_level = check_for_higher_speed_level(i + 1);

					// Search the temp window for the actual temp
					// If the previous temp was lower, than the upper temp boundary is the normal "temp on" value of the next window
					// If the previous temp was higher, than the upper temp boundary is temp off value of the next window

					if (   (temp >= on_temp[i]) &&
						 (((temp < on_temp[i + 1]) && !previous_level) ||
						  ((temp <= off_temp[i + 1]) && previous_level )) )
					{
						// If just the right fan speed? Than, no need to set another time
						if (speed[i] != actual_level)
						{
							result = ACPI.METHOD.SetValue("_SB.PCI0.SBRG.EC0.WMFN", ((speed[i] + 6) << 3) | 0x07);
							actual_level = speed[i];
						}
						// Break the search
						find = true;
					}
					i++;
				}

                return result;
            }


        }
        //
        #endregion

    }
    //
    #endregion

}